@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2021 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -35,23 +35,20 @@
 
 import sys
 import queue
-import platform
 import threading
 
 from pyglet import app
 from pyglet import clock
 from pyglet import event
-from pyglet import compat_platform
 
 _is_pyglet_doc_run = hasattr(sys, "is_pyglet_doc_run") and sys.is_pyglet_doc_run
 
 
 class PlatformEventLoop:
     """ Abstract class, implementation depends on platform.
-    
+
     .. versionadded:: 1.2
     """
-
     def __init__(self):
         self._event_queue = queue.Queue()
         self._is_running = threading.Event()
@@ -72,8 +69,8 @@ class PlatformEventLoop:
         is able to dispatch the event.  This method can be safely called
         from any thread.
 
-        If the method is called from the :py:meth:`run` method's thread (for 
-        example, from within an event handler), the event may be dispatched 
+        If the method is called from the :py:meth:`run` method's thread (for
+        example, from within an event handler), the event may be dispatched
         within the same runloop iteration or the next one; the choice is
         nondeterministic.
 
@@ -120,7 +117,7 @@ class PlatformEventLoop:
         raise NotImplementedError('abstract')
 
     def set_timer(self, func, interval):
-        raise NotImplementedError('abstract')
+        pass
 
     def stop(self):
         pass
@@ -130,7 +127,7 @@ class EventLoop(event.EventDispatcher):
     """The main run loop of the application.
 
     Calling `run` begins the application event loop, which processes
-    operating system events, calls :py:func:`pyglet.clock.tick` to call 
+    operating system events, calls :py:func:`pyglet.clock.tick` to call
     scheduled functions and calls :py:meth:`pyglet.window.Window.on_draw` and
     :py:meth:`pyglet.window.Window.flip` to update window contents.
 
@@ -149,7 +146,16 @@ class EventLoop(event.EventDispatcher):
         self.clock = clock.get_default()
         self.is_running = False
 
-    def run(self):
+    @staticmethod
+    def _redraw_windows(dt):
+        # Redraw all windows
+        for window in app.windows:
+            window.switch_to()
+            window.dispatch_event('on_draw')
+            window.dispatch_event('on_refresh', dt)
+            window.flip()
+
+    def run(self, interval=1/60):
         """Begin processing events, scheduled functions and window updates.
 
         This method returns when :py:attr:`has_exit` is set to True.
@@ -157,8 +163,20 @@ class EventLoop(event.EventDispatcher):
         Developers are discouraged from overriding this method, as the
         implementation is platform-specific.
         """
+        if not interval:
+            self.clock.schedule(self._redraw_windows)
+        else:
+            self.clock.schedule_interval(self._redraw_windows, interval)
+
         self.has_exit = False
-        self._legacy_setup()
+
+        from pyglet.window import Window
+        Window._enable_event_queue = False
+
+        # Dispatch pending events
+        for window in app.windows:
+            window.switch_to()
+            window.dispatch_pending_events()
 
         platform_event_loop = app.platform_event_loop
         platform_event_loop.start()
@@ -173,16 +191,6 @@ class EventLoop(event.EventDispatcher):
         self.dispatch_event('on_exit')
         platform_event_loop.stop()
 
-    def _legacy_setup(self):
-        # Disable event queuing for dispatch_events
-        from pyglet.window import Window
-        Window._enable_event_queue = False
-
-        # Dispatch pending events
-        for window in app.windows:
-            window.switch_to()
-            window.dispatch_pending_events()
-
     def enter_blocking(self):
         """Called by pyglet internal processes when the operating system
         is about to block due to a user interaction.  For example, this
@@ -200,7 +208,8 @@ class EventLoop(event.EventDispatcher):
         timeout = self.idle()
         app.platform_event_loop.set_timer(self._blocking_timer, timeout)
 
-    def exit_blocking(self):
+    @staticmethod
+    def exit_blocking():
         """Called by pyglet internal processes when the blocking operation
         completes.  See :py:meth:`enter_blocking`.
         """
@@ -236,15 +245,7 @@ class EventLoop(event.EventDispatcher):
             be called again, or `None` to block for user input.
         """
         dt = self.clock.update_time()
-        redraw_all = self.clock.call_scheduled_functions(dt)
-
-        # Redraw all windows
-        for window in app.windows:
-            if redraw_all or (window._legacy_invalid and window.invalid):
-                window.switch_to()
-                window.dispatch_event('on_draw')
-                window.flip()
-                window._legacy_invalid = False
+        self.clock.call_scheduled_functions(dt)
 
         # Update timout
         return self.clock.get_sleep_time(True)
@@ -275,7 +276,7 @@ class EventLoop(event.EventDispatcher):
     def exit(self):
         """Safely exit the event loop at the end of the current iteration.
 
-        This method is a thread-safe equivalent for for setting 
+        This method is a thread-safe equivalent for setting
         :py:attr:`has_exit` to ``True``.  All waiting threads will be
         interrupted (see :py:meth:`sleep`).
         """
