@@ -94,6 +94,10 @@ def dump_pyglet():
 
 def dump_window():
     """Dump display, window, screen and default config info."""
+    from pyglet.gl import gl_info
+    if not gl_info.have_version(3):
+        print(f"Insufficient OpenGL version: {gl_info.get_version_string()}")
+        return
     import pyglet.window
     display = pyglet.canvas.get_display()
     print('display:', repr(display))
@@ -126,17 +130,6 @@ def dump_gl(context=None):
         print('  ', name)
 
 
-def dump_glu():
-    """Dump GLU info."""
-    from pyglet.gl import glu_info
-    print('glu_info.get_version():', glu_info.get_version())
-    print('glu_info.get_extensions():')
-    extensions = list(glu_info.get_extensions())
-    extensions.sort()
-    for name in extensions:
-        print('  ', name)
-
-
 def dump_glx():
     """Dump GLX info."""
     try:
@@ -234,7 +227,6 @@ def dump():
     _try_dump('Python', dump_python)
     _try_dump('pyglet', dump_pyglet)
     _try_dump('pyglet.window', dump_window)
-    _try_dump('pyglet.gl.glu_info', dump_glu)
     _try_dump('pyglet.gl.glx_info', dump_glx)
     _try_dump('pyglet.media', dump_media)
     _try_dump('pyglet.media.ffmpeg', dump_ffmpeg)
