@@ -230,9 +230,9 @@ class Vec2:
         :returns: A new rotated vector of the same magnitude.
         :rtype: Vec2
         """
-        mag = self.mag
-        heading = self.heading
-        return Vec2(mag * _math.cos(heading + angle), mag * _math.sin(heading + angle))
+        s = _math.sin(angle)
+        c = _math.cos(angle)
+        return Vec2(c * self.x - s * self.y, s * self.x + c * self.y)
 
     def distance(self, other: Vec2) -> float:
         """Calculate the distance between this vector and another 2D vector."""
@@ -456,10 +456,11 @@ class Vec3:
         :returns: A unit vector with the same rotation.
         :rtype: Vec3
         """
-        d = self.__abs__()
-        if d:
+        try:
+            d = self.__abs__()
             return Vec3(self.x / d, self.y / d, self.z / d)
-        return self
+        except ZeroDivisionError:
+            return self
 
     def clamp(self, min_val: float, max_val: float) -> Vec3:
         """Restrict the value of the X,  Y and Z components of the vector to be within the given values.
@@ -818,6 +819,31 @@ class Mat4(tuple):
                     0, 0, q, -1,
                     0, 0, qn, 0))
 
+    @classmethod
+    def from_rotation(cls, angle: float, vector: Vec3) -> Mat4:
+        """Create a rotation matrix from an angle and Vec3.
+
+        :Parameters:
+            `angle` : A `float` :
+                The angle as a float.
+            `vector` : A `Vec3`, or 3 component tuple of float or int :
+                Vec3 or tuple with x, y and z translation values
+        """
+        return cls().rotate(angle, vector)
+
+    @classmethod
+    def from_scale(cls: type[Mat4T], vector: Vec3) -> Mat4T:
+        """Create a scale matrix from a Vec3.
+
+        :Parameters:
+            `vector` : A `Vec3`, or 3 component tuple of float or int
+                Vec3 or tuple with x, y and z scale values
+        """
+        return cls((vector[0], 0.0, 0.0, 0.0,
+                    0.0, vector[1], 0.0, 0.0,
+                    0.0, 0.0, vector[2], 0.0,
+                    0.0, 0.0, 0.0, 1.0))
+
     @classmethod
     def from_translation(cls: type[Mat4T], vector: Vec3) -> Mat4T:
         """Create a translation matrix from a Vec3.
@@ -831,18 +857,6 @@ class Mat4(tuple):
                     0.0, 0.0, 1.0, 0.0,
                     vector[0], vector[1], vector[2], 1.0))
 
-    @classmethod
-    def from_rotation(cls, angle: float, vector: Vec3) -> Mat4:
-        """Create a rotation matrix from an angle and Vec3.
-
-        :Parameters:
-            `angle` : A `float` :
-                The angle as a float.
-            `vector` : A `Vec3`, or 3 component tuple of float or int :
-                Vec3 or tuple with x, y and z translation values
-        """
-        return cls().rotate(angle, vector)
-
     @classmethod
     def look_at_direction(cls: type[Mat4T], direction: Vec3, up: Vec3) -> Mat4T:
         vec_z = direction.normalize()
@@ -855,11 +869,16 @@ class Mat4(tuple):
                     0.0, 0.0, 0.0, 1.0))
 
     @classmethod
-    def look_at(cls, position: Vec3, target: Vec3, up: Vec3) -> Mat4:
-        direction = target - position
-        direction_mat4 = cls.look_at_direction(direction, up)
-        position_mat4 = cls.from_translation(-position)
-        return direction_mat4 @ position_mat4
+    def look_at(cls: type[Mat4T], position: Vec3, target: Vec3, up: Vec3):
+        f = (target - position).normalize()
+        u = up.normalize()
+        s = f.cross(u)
+        u = s.cross(f)
+
+        return cls([s.x, u.x, -f.x, 0.0,
+                    s.y, u.y, -f.y, 0.0,
+                    s.z, u.z, -f.z, 0.0,
+                    -s.dot(position), -u.dot(position), f.dot(position), 1.0])
 
     def row(self, index: int) -> tuple:
         """Get a specific row as a tuple."""
@@ -869,18 +888,6 @@ class Mat4(tuple):
         """Get a specific column as a tuple."""
         return self[index::4]
 
-    def scale(self, vector: Vec3) -> Mat4:
-        """Get a scale Matrix on x, y, or z axis."""
-        temp = list(self)
-        temp[0] *= vector[0]
-        temp[5] *= vector[1]
-        temp[10] *= vector[2]
-        return Mat4(temp)
-
-    def translate(self, vector: Vec3) -> Mat4:
-        """Get a translation Matrix along x, y, and z axis."""
-        return self @ Mat4((1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, *vector, 1))
-
     def rotate(self, angle: float, vector: Vec3) -> Mat4:
         """Get a rotation Matrix on x, y, or z axis."""
         if not all(abs(n) <= 1 for n in vector):
@@ -908,6 +915,18 @@ class Mat4(tuple):
 
         return Mat4(self) @ Mat4((ra, rb, rc, 0, re, rf, rg, 0, ri, rj, rk, 0, 0, 0, 0, 1))
 
+    def scale(self, vector: Vec3) -> Mat4:
+        """Get a scale Matrix on x, y, or z axis."""
+        temp = list(self)
+        temp[0] *= vector[0]
+        temp[5] *= vector[1]
+        temp[10] *= vector[2]
+        return Mat4(temp)
+
+    def translate(self, vector: Vec3) -> Mat4:
+        """Get a translation Matrix along x, y, and z axis."""
+        return self @ Mat4((1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, *vector, 1))
+
     def transpose(self) -> Mat4:
         """Get a transpose of this Matrix."""
         return Mat4(self[0::4] + self[1::4] + self[2::4] + self[3::4])
